@@ -4,18 +4,6 @@
 #
 # SPDX-License-Identifier: Apache-2.0
 #
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-#
 # Author(s):    Ryan Pavlik <ryan.pavlik@collabora.com>
 """Provides utilities to write a script to verify XML registry consistency."""
 
@@ -117,7 +105,7 @@ class XMLChecker:
                                ', '.join(unrecognized))
 
         self.referenced_input_types = ReferencedTypes(self.db, self.is_input)
-        self.referenced_api_types = ReferencedTypes(self.db, self.is_api_type)
+        self.referenced_types = ReferencedTypes(self.db)
         if not suppressions:
             suppressions = {}
         self.suppressions = DictOfStringSets(suppressions)
@@ -182,7 +170,8 @@ class XMLChecker:
         be skipped for a command.
 
         May override."""
-        return False
+
+        return self.conventions.should_skip_checking_codes
 
     def get_codes_for_command_and_type(self, cmd_name, type_name):
         """Return a set of error codes expected due to having
@@ -212,12 +201,11 @@ class XMLChecker:
 
             self.check_type(name, info, cat)
 
+        self.ext_numbers = set()
         for name, info in self.reg.extdict.items():
-            if info.elem.get('supported') != self.conventions.xml_api_name:
-                # Skip unsupported extensions
-                continue
+            supported = (info.elem.get('supported') == self.conventions.xml_api_name)
             self.set_error_context(entity=name, elem=info.elem)
-            self.check_extension(name, info)
+            self.check_extension(name, info, supported)
 
         entities_with_messages = set(
             self.errors.keys()).union(self.warnings.keys())
@@ -298,6 +286,22 @@ class XMLChecker:
                         self.record_error("References a non-existent parameter/member in the length of",
                                           getElemName(param), ":", entry.other_param_name)
 
+    def check_referenced_type(self, desc, ref_name):
+        """
+        Record an error if a type mentioned somewhere doesn't exist.
+
+        :param desc: Description of where this type reference was found,
+                     for the error messsage.
+        :param ref_name: The name of the referenced type. If false-ish (incl. None),
+                         checking is skipped, so OK to pass the results of
+                         info.elem.get() directly
+        """
+        if ref_name:
+            entity = self.db.findEntity(ref_name)
+            if not entity:
+                self.record_error("Unknown type named in", desc, ":",
+                                  ref_name)
+
     def check_type(self, name, info, category):
         """Check a type's XML data for consistency.
 
@@ -323,17 +327,32 @@ class XMLChecker:
                         self.record_error("Type has incorrect type-member value: expected",
                                           expected, "got", val)
 
+            # Check structextends attribute, if present.
+            self.check_referenced_type("'structextends' attribute", info.elem.get("structextends"))
+
+            # Check parentstruct attribute, if present.
+            self.check_referenced_type("'parentstruct' attribute", info.elem.get("parentstruct"))
+
         elif category == "bitmask":
             if 'Flags' not in name:
                 self.record_error("Name of bitmask doesn't include 'Flags'")
+        elif category == "handle":
+            # Check parent attribute, if present.
+            self.check_referenced_type("'parent' attribute", info.elem.get("parent"))
 
-    def check_extension(self, name, info):
+    def check_extension(self, name, info, supported):
         """Check an extension's XML data for consistency.
 
         Called from check.
 
         May extend."""
-        pass
+        # Verify that each extension has a unique number
+        extension_number = info.elem.get('number')
+        if extension_number is not None and extension_number != '0':
+            if extension_number in self.ext_numbers:
+                self.record_error('Duplicate extension number ' + extension_number)
+            else:
+                self.ext_numbers.add(extension_number)
 
     def check_command(self, name, info):
         """Check a command's XML data for consistency.
@@ -413,7 +432,7 @@ class XMLChecker:
 
         May extend."""
         referenced_input = self.referenced_input_types[name]
-        referenced_types = self.referenced_api_types[name]
+        referenced_types = self.referenced_types[name]
 
         # Check that we have all the codes we expect, based on input types.
         for referenced_type in referenced_input:
