@@ -4,18 +4,6 @@
 #
 # SPDX-License-Identifier: Apache-2.0
 #
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-#
 # Author(s):    Ryan Pavlik <ryan.pavlik@collabora.com>
 
 import logging
@@ -100,7 +88,9 @@ class Attrib(Enum):
     REFPAGE = 'refpage'
     DESC = 'desc'
     TYPE = 'type'
+    ALIAS = 'alias'
     XREFS = 'xrefs'
+    ANCHOR = 'anchor'
 
 
 VALID_REF_PAGE_ATTRIBS = set(
@@ -758,8 +748,9 @@ class MacroCheckerFile(object):
 
         # OK, so we don't recognize this entity (and couldn't auto-fix it).
 
-        if self.checker.isLinkedMacro(macro):
-            # This should be linked, which means we should know the target.
+        if self.checker.entity_db.shouldBeRecognized(macro, entity):
+            # We should know the target - it's a link macro,
+            # or there's some reason the entity DB thinks we should know it.
             if self.checker.likelyRecognizedEntity(entity):
                 # Should be linked and it matches our pattern,
                 # so probably not wrong macro.
@@ -837,6 +828,10 @@ class MacroCheckerFile(object):
                     self.error(MessageId.WRONG_MACRO, msg,
                                group='macro', replacement=data.macro, fix=replacement)
             else:
+                if self.checker.likelyRecognizedEntity(entity):
+                    # This is a use of *text: for something that fits the pattern but isn't in the spec.
+                    # This is OK.
+                    return False
                 msg.append('Entity not found in spec, either.')
                 if macro[0] != 'e':
                     # Only suggest a macro if we aren't in elink/ename/etext,
@@ -1032,6 +1027,13 @@ class MacroCheckerFile(object):
                        "Found apparent reference page markup, but missing type='...'",
                        group=None)
 
+        # Optional field: alias='spaceDelimited validEntities'
+        # Currently does nothing. Could modify checkRefPageXrefs to also
+        # check alias= attribute value
+        # if Attrib.ALIAS.value in attribs:
+        #    # This field is optional
+        #    self.checkRefPageXrefs(attribs[Attrib.XREFS.value])
+
         # Optional field: xrefs='spaceDelimited validEntities'
         if Attrib.XREFS.value in attribs:
             # This field is optional
