@@ -1,20 +1,8 @@
 #!/usr/bin/python3 -i
 #
-# Copyright (c) 2013-2020 The Khronos Group Inc.
+# Copyright (c) 2013-2021, The Khronos Group Inc.
 #
 # SPDX-License-Identifier: Apache-2.0
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
 
 from collections import OrderedDict, namedtuple
 from enum import Enum
@@ -1375,6 +1363,19 @@ class ValidityOutputGenerator(OutputGenerator):
         self.writeInclude('protos', name, validity, threadsafety,
                           commandpropertiesentry, successcodes, errorcodes)
 
+    def genStructInternals(self, typeinfo, typeName, validity, threadsafety):
+        """Internals of generating for a struct, so aliases can be handled."""
+
+        if typeinfo.elem.get('returnedonly') is None:
+            validity += self.makeStructOrCommandValidity(
+                typeinfo.elem, typeName, typeinfo.getMembers())
+            threadsafety = self.makeThreadSafetyBlock(typeinfo.elem, 'member')
+
+        else:
+            # Need to generate structure type and next pointer chain member validation
+            validity += self.makeOutputOnlyStructValidity(
+                typeinfo.elem, typeName, typeinfo.getMembers())
+
     def genStruct(self, typeinfo, typeName, alias):
         """Struct Generation."""
         OutputGenerator.genStruct(self, typeinfo, typeName, alias)
@@ -1388,15 +1389,14 @@ class ValidityOutputGenerator(OutputGenerator):
         # OpenXR-only: make sure extension is enabled
         validity.possiblyAddExtensionRequirement(self.currentExtension, 'using slink:')
 
-        if typeinfo.elem.get('returnedonly') is None:
-            validity += self.makeStructOrCommandValidity(
-                typeinfo.elem, typeName, typeinfo.getMembers())
-            threadsafety = self.makeThreadSafetyBlock(typeinfo.elem, 'member')
+        self.genStructInternals(typeinfo, typeName, validity, threadsafety)
 
-        else:
-            # Need to generate structure type and next pointer chain member validation
-            validity += self.makeOutputOnlyStructValidity(
-                typeinfo.elem, typeName, typeinfo.getMembers())
+        if alias and self.conventions.duplicate_aliased_structs:
+            validity.addValidityEntry(
+                '**Note:** slink:{new} is an alias for slink:{old}, so implicit valid usage for slink:{old} has been replicated below.'.format(
+                    old=alias, new=typeName))
+            alias_info = self.registry.typedict[alias]
+            self.genStructInternals(alias_info, alias, validity, threadsafety)
 
         self.writeInclude('structs', typeName, validity,
                           threadsafety, None, None, None)
