@@ -1,22 +1,10 @@
-// Copyright (c) 2017-2020 The Khronos Group Inc.
+// Copyright (c) 2017-2021, The Khronos Group Inc.
 // Copyright (c) 2017-2019 Valve Corporation
 // Copyright (c) 2017-2019 LunarG, Inc.
 //
-// SPDX-License-Identifier: Apache-2.0
+// SPDX-License-Identifier: Apache-2.0 OR MIT
 //
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
-//
-// Author: Mark Young <marky@lunarg.com>
+// Initial Author: Mark Young <marky@lunarg.com>
 //
 
 #include "api_layer_interface.hpp"
@@ -48,7 +36,7 @@ static void AddEnvironmentApiLayers(std::vector<std::string>& enabled_layers) {
 
     // Handle any path listings in the string (separated by the appropriate path separator)
     while (found != std::string::npos) {
-        cur_search = layers.substr(last_found, found);
+        cur_search = layers.substr(last_found, found - last_found);
         enabled_layers.push_back(cur_search);
         last_found = found + 1;
         found = layers.find_first_of(PATH_SEPARATOR, last_found);
@@ -235,19 +223,15 @@ XrResult ApiLayerInterface::LoadApiLayers(const std::string& openxr_command, uin
     }
 
     for (std::unique_ptr<ApiLayerManifestFile>& manifest_file : layer_manifest_files) {
-        bool enabled = false;
-
         // Always add implicit layers.  They would only be in this list if they were enabled
         // (i.e. the disable environment variable is not set).
-        if (manifest_file->Type() == MANIFEST_TYPE_IMPLICIT_API_LAYER) {
-            enabled = true;
-        } else {
-            // Only add explicit layers if they are called out by the application
-            for (uint32_t layer = 0; layer < enabled_api_layers.size(); ++layer) {
-                if (enabled_api_layers[layer] == manifest_file->LayerName()) {
-                    layer_found[layer] = true;
-                    enabled = true;
-                }
+        bool enabled = (manifest_file->Type() == MANIFEST_TYPE_IMPLICIT_API_LAYER);
+
+        // Check if this layer is explicitly enabled.
+        for (uint32_t layer = 0; layer < enabled_api_layers.size(); ++layer) {
+            if (enabled_api_layers[layer] == manifest_file->LayerName()) {
+                layer_found[layer] = true;
+                enabled = true;
             }
         }
 
